import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

class OutOfRangeException extends Exception
{
	// nothing
}

public class GameOption extends JDialog implements ActionListener {
	
	public static final int minRows = 5;
	public static final int maxRows = 30;
	public static final int minCols = 5;
	public static final int maxCols = 30;
	public static final int minBallTypes = 2;
	public static final int maxBallTypes = 5;
	
	private static int NumOfRows = 7;
	private static int NumOfCols = 16;
	private static int NumOfBallTypes = 3;

	private JTextField txtRows, txtCols, txtTypes;

	public GameOption(Frame owner, boolean modal) {
		super(owner, modal);
		// ...
		setSize(260, 185);
		setLocationRelativeTo(owner);
		setResizable(false);
		addWindowListener(new WindowCloser(this));

		setLayout(null);
		setTitle("SameGame Options");
		
		JLabel l = new JLabel("Number of rows (" + minRows + " - " + maxRows + ") :");
		l.setBounds(10, 10, 180, 20);
		add(l);
		l = new JLabel("Number of columns (" + minCols + " - " + maxCols + ") :");
		l.setBounds(10, 40, 180, 20);
		add(l);
		l = new JLabel("Number of ball types (" + minBallTypes + " - " + maxBallTypes + ") :");
		l.setBounds(10, 70, 180, 20);
		add(l);
		l = new JLabel("* Changes are applied at next game.");
		l.setBounds(10, 95, 240, 20);
		l.setVerticalTextPosition(JLabel.TOP);
		add(l);
		txtRows = new JTextField(String.valueOf(NumOfRows));
		txtRows.setBounds(200, 10, 40, 20);
		add(txtRows);
		txtCols = new JTextField(String.valueOf(NumOfCols));
		txtCols.setBounds(200, 40, 40, 20);
		add(txtCols);
		txtTypes = new JTextField(String.valueOf(NumOfBallTypes));
		txtTypes.setBounds(200, 70, 40, 20);
		add(txtTypes);

		JButton b = new JButton("OK");
		b.setBounds(40, 120, 80, 25);
		b.addActionListener(this);
		add(b);
		this.getRootPane().setDefaultButton(b);
		b = new JButton("Cancel");
		b.setBounds(130, 120, 80, 25);
		b.addActionListener(this);
		add(b);
	}

	public static int getNumOfRows() { return NumOfRows; }
	public static int getNumOfCols() { return NumOfCols; }
	public static int getNumOfBallTypes() { return NumOfBallTypes; }

	public static void setNumOfRows(int rows) throws OutOfRangeException {
		if (rows < minRows || rows > maxRows) throw new OutOfRangeException();
		NumOfRows = rows;
	}

	public static void setNumOfCols(int cols) throws OutOfRangeException {
		if (cols < minCols || cols > maxCols) throw new OutOfRangeException();
		NumOfCols = cols;
	}

	public static void setNumOfBallTypes(int t) throws OutOfRangeException {
		if (t < minBallTypes || t > maxBallTypes) throw new OutOfRangeException();
		NumOfBallTypes = t;
	}

	public static void load() {
		// not implemented.
	}

	public static void save() {
		// not implemented.
	}

	// -----------------------------------------------------------------------
	
	static class WindowCloser extends java.awt.event.WindowAdapter {
		private Window iWindow;
		public WindowCloser(Window w) { iWindow = w; }
		
	    public void windowClosing(WindowEvent e) {
			iWindow.setVisible(false);
			iWindow.dispose();
		}
	}
		
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand() == "OK") {
			try {
				setNumOfRows(Integer.parseInt(txtRows.getText()));
				setNumOfCols(Integer.parseInt(txtCols.getText()));
				setNumOfBallTypes(Integer.parseInt(txtTypes.getText()));
			} catch (NumberFormatException err) {
				JOptionPane.showMessageDialog(this, "Wrong number. Check your input.", "SameGame", JOptionPane.ERROR_MESSAGE);
				return;
			} catch (OutOfRangeException err) {
				JOptionPane.showMessageDialog(this, "Out of range.", "SameGame", JOptionPane.ERROR_MESSAGE);
				return;
			}
			this.setVisible(false);
			this.dispose();
		} else if (e.getActionCommand() == "Cancel") {
			this.setVisible(false);
			this.dispose();
		}
	}

}
