import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.io.*;
import java.util.*;

// Ranking class:
// This implements the ranking dialog and some untility methods for ranking.
public class Ranking extends JDialog implements ActionListener {
	private static java.util.List<Record> records = new ArrayList<Record>();
	private static Set<RecordType> recordTypes = new HashSet<RecordType>();

	private JComboBox cboType;
	private JTable tblList;
	private DefaultTableModel iTableModel;

	public Ranking(Frame owner, boolean modal) {
		super(owner, modal);
		
		setSize(310, 415);
		setLayout(null);
		setLocationRelativeTo(owner);
		setResizable(false);
		setTitle("SameGame Ranking");
		addWindowListener(new WindowCloser(this));

		JLabel l = new JLabel("Type of settings : ");
		l.setBounds(10, 10, 120, 20);
		add(l);

		cboType = new JComboBox();
		cboType.setEditable(false);
		cboType.setBounds(130, 10, 120, 20);
		cboType.addActionListener(new cbpTypeSelector(this));
		add(cboType);

		tblList = new JTable();
		JScrollPane s = new JScrollPane(tblList);
		s.setBounds(10, 40, 280, 300);
		add(s);
		
		JButton b = new JButton("Close");
		b.setBounds(100, 350, 100, 25);
		b.addActionListener(this);
		add(b);
		this.getRootPane().setDefaultButton(b);

		// --------

		Iterator<RecordType> it = recordTypes.iterator();
		while (it.hasNext())
			cboType.addItem(it.next());
	}

	// --------------------------------------------------------
	
	// check if best
	public static boolean isNewRecord(Record rec) {
		Iterator<Record> it = records.iterator();
		SortedSet<Record> rset = new TreeSet<Record>();
		while (it.hasNext()) {
			Record r = it.next();
			if (r.isType(rec.getType()))
				rset.add(r);
		}
		if (rset.size() == 0) return true; // no records -> current is new record
		return (rec.compareTo(rset.first()) < 0);
	}

	public static void addRecord(Record rec) {
		records.add(rec);
		recordTypes.add(rec.getType());
	}

	public static void clearRecords() {
		records.clear();
		recordTypes.clear();
	}

	// save the records to file
	public static void save() {
		File f = new File("ranking.txt");
		String line;
		if (f.canWrite()) {
			try {
				BufferedWriter writer = new BufferedWriter(new FileWriter(f));
				Iterator<Record> it = records.iterator();
				while (it.hasNext()) {
					Record r = it.next();
					writer.write(r.getName(), 0, r.getName().length());
					writer.newLine();
					line = r.getRows() + "," + r.getCols() + "," + r.getBallTypes() + "," + r.getScore();
					writer.write(line, 0, line.length());
					writer.newLine();
				}
				writer.close();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Cannot save yout game records.", "SameGame", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
	}

	// load the records from file
	public static void load() {
		File f = new File("ranking.txt");
		String line, name = "";
		String[] args;
		int i = 0;
		if (f.exists() && f.canRead()) {
			clearRecords();
			try {
				BufferedReader reader = new BufferedReader(new FileReader(f));
				while ((line = reader.readLine()) != null) {
					if (i % 2 == 0)
						name = line;
					else {
						args = line.split(",");
						addRecord(new Record(Integer.parseInt(args[0]), Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), name));
					}
					++i;
				}
				reader.close();
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Error while reading record file.", "SameGame", JOptionPane.ERROR_MESSAGE);
				clearRecords();
				return;
			}
		}
	}

	// ---------------------------------------------------------------------
	
	static class WindowCloser extends java.awt.event.WindowAdapter {
		private Window iWindow;
		public WindowCloser(Window w) { iWindow = w; }
		
	    public void windowClosing(WindowEvent e) {
			iWindow.setVisible(false);
			iWindow.dispose();
		}
	}

	static class cbpTypeSelector implements ActionListener {
		private Ranking iParent;
		public cbpTypeSelector(Ranking r) { iParent = r; }

		public void actionPerformed(ActionEvent e) {
			int c = 0;
			iParent.iTableModel = new DefaultTableModel();
			SortedSet<Record> rset = new TreeSet<Record>();
			Iterator<Record> it = iParent.records.iterator();
			// get the records of current type
			RecordType current = (RecordType)iParent.cboType.getSelectedItem();
			while (it.hasNext()) {
				Record r = it.next();
				if (r.isType(current)) {
					rset.add(r);
					++c;
				}
			}
			// add to data
			String[][] data = new String[c][3];
			it = rset.iterator(); c = 0;
			while (it.hasNext()) {
				Record r = it.next();
				data[c][0] = r.getName();
				data[c][1] = r.getType().toString();
				data[c][2] = String.valueOf(r.getScore());
				++c;
			}
			// set the table
			String[] colNames = {"Name","Game Type","Score"};
			iParent.iTableModel.setDataVector(data, colNames);
			iParent.tblList.setModel(iParent.iTableModel);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "Close") {
			this.setVisible(false);
			this.dispose();
		}
	}
}
