
// Record class: 
// Specifies the record of a game with its setting, score and name.
public class Record implements Comparable {

	private int iRows, iCols, iScore, iTypes;
	private String iName;

	private Record() {
		// do nothing
	}

	public Record(int rows, int cols, int ballTypes, int score) {
		iRows = rows;
		iCols = cols;
		iScore = score;
		iTypes = ballTypes;
	}

	public Record(int rows, int cols, int ballTypes, int score, String name) {
		this(rows, cols, ballTypes, score);
		iName = name;
	}

	public String getName() { return iName; }
	public void setName(String name) { iName = name; }
	public int getRows() { return iRows; }
	public int getCols() { return iCols; }
	public int getBallTypes() { return iTypes; }
	public int getScore() { return iScore; }

	public RecordType getType() {
		return new RecordType(iRows, iCols, iTypes);
	}

	public boolean isType(RecordType t) {
		return getType().equals(t);
	}

	// implementation of Comprable<Record> (for SortedSet)
	public int compareTo(Object rhs) throws ClassCastException {
		if (!(rhs instanceof Record)) throw new ClassCastException();
		if (getType().equals(((Record)rhs).getType())) {
			return (iScore > ((Record)rhs).iScore) ? -1 : ((iScore == ((Record)rhs).iScore) ? 0 : 1);
		} else {
			// not comparable!
			throw new ClassCastException("Not comparable by record types.");
		}
	}
	
}

// RecordType class:
// To be fair with ranking, we must distinguish game settings.
// This class is used for HashSet.
class RecordType {
	public int rows, cols, ballTypes;

	public RecordType() { }
	public RecordType(Record r) {
		rows = r.getRows();
		cols = r.getCols();
		ballTypes = r.getBallTypes();
	}
	public RecordType(int r, int c, int b) {
		rows = r;
		cols = c;
		ballTypes = b;
	}

	public boolean equals(Object rhs) {
		if (rhs == null)
			return false;
		else if (rhs instanceof RecordType)
			return hashCode() == rhs.hashCode();
		else
			return false;
	}

	public int hashCode() {
		return rows * 1000 + cols * 10 + ballTypes;
	}

	public String toString() {
		return rows + " x " + cols + " / " + ballTypes;
	}
}
