import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.*;

public class SameGame extends JFrame implements ActionListener {

  private Board iBoard;
  private int iPoints;
  private boolean isPlaying = false;

  private final String MENU_STARTNEW = "Start new game";
  private final String MENU_RANKING = "Ranking...";
  private final String MENU_OPTION = "Options...";
  private final String MENU_EXIT = "Exit";
  private final String MENU_ABOUT = "About this game...";

  private myCanvas canvas;
  
  public SameGame(Board board)
  {
    // initialization
    iBoard = board;
    iPoints = 0;
    setSize(740, 460);
    setLocationRelativeTo(null);
    canvas = new myCanvas(this);
    getContentPane().add(canvas);
    
    // menu creation 
    JMenu mnuGame = new JMenu("Game");
    mnuGame.add(MENU_STARTNEW);
    mnuGame.add(MENU_RANKING);
    mnuGame.add(MENU_OPTION);
    mnuGame.addSeparator();
    mnuGame.add(MENU_EXIT);
    JMenu mnuAbout = new JMenu("Help");
    mnuAbout.add(MENU_ABOUT);
    JMenuBar menu = new JMenuBar();
    menu.add(mnuGame);
    menu.add(mnuAbout);
    setJMenuBar(menu);
    
    // event handlers
    addWindowListener(new AppCloser());
    addComponentListener(new Resizer(this));
    mnuGame.getItem(0).addActionListener(this);
    mnuGame.getItem(1).addActionListener(this);
    mnuGame.getItem(2).addActionListener(this);
    mnuGame.getItem(4).addActionListener(this);
    mnuAbout.getItem(0).addActionListener(this);
    
    // update
    doUpdate();
    isPlaying = true;
  }

  private void doUpdate()
  {
    setTitle("SameGame - " + iPoints + " points");
    canvas.repaint();
  }

  // Start new game
  private void newGame()
  {
    iPoints = 0;
    iBoard = new Board(GameOption.getNumOfRows(), GameOption.getNumOfCols(), GameOption.getNumOfBallTypes());
    canvas.resetHighlights();
    canvas.randomizeBalls();
    doUpdate();
    isPlaying = true;
  }

  // Finish a game
  private void finishGame()
  {
    isPlaying = false;
    // make a record
    Record r = new Record(iBoard.rows(), iBoard.columns(), iBoard.types(), iPoints);
    if (Ranking.isNewRecord(r)) {
      String name = JOptionPane.showInputDialog(this, "New Record! (" + iPoints +" points)\nPlease input your name.", "Congratulations!");
      if (name != null) {
	r.setName(name);
	Ranking.addRecord(r);
      }
    } else {
      JOptionPane.showMessageDialog(this, "Game Over!\nYour score is " + iPoints + ".", "SameGame", JOptionPane.INFORMATION_MESSAGE);
    }
    if (JOptionPane.showConfirmDialog(this, "New game now?", "SameGame", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
      newGame();
  }

  private static void doExit()
  {
    if (JOptionPane.showConfirmDialog(null, "Exit now?", "SameGame", JOptionPane.OK_CANCEL_OPTION)
	== JOptionPane.OK_OPTION) {
      GameOption.save();
      Ranking.save();
      System.exit(0);
    }
  }

  // --------------------------------------------------------------------

  static class AppCloser extends java.awt.event.WindowAdapter {
    public void windowClosing(WindowEvent e) {
      doExit();
    }
  }

  static class Resizer extends java.awt.event.ComponentAdapter {
    public Resizer(SameGame g) { iParent = g; }

    public void componentResized(ComponentEvent e)
    {
      iParent.doUpdate();
    }
    private SameGame iParent;
  }
  
  public void actionPerformed(ActionEvent e)
  {
    // menu event handler
    if (e.getActionCommand() == MENU_EXIT) {
      doExit();
    } else if (e.getActionCommand() == MENU_ABOUT) {
      JOptionPane.showMessageDialog(null, "SameGame (for Data Structure programming project 2)\n\n" +
	  "Programmed by Daybreaker(Joon-gi Kim) and Otfried Cheong.\n" +
	  "Distributed by GNU General Public License.\n" + 
	  "Block images from BEJEWELED.");
    } else if (e.getActionCommand() == MENU_RANKING) {
      Ranking rankingdlg = new Ranking(this, true);
      rankingdlg.setVisible(true);
    } else if (e.getActionCommand() == MENU_OPTION) {
      GameOption optiondlg = new GameOption(this, true);
      optiondlg.setVisible(true);
    } else if (e.getActionCommand() == MENU_STARTNEW) {
      if (JOptionPane.showConfirmDialog(null, "Restart now?", "SameGame", JOptionPane.OK_CANCEL_OPTION)
	  == JOptionPane.OK_OPTION) {
	newGame();
      }
    }
  }

  // --------------------------------------------------------------------

  // myCanvas do all actual drawings, separated to SameGame
  static class myCanvas extends JPanel {

    private SameGame iParent;
    private int iPrevRow = -1;
    private int iPrevCol = -1;
    private static Color COLOR_BACKGROUND = new Color(49, 49, 41);
    private static Color COLOR_HIGHLIGHTED_BACKGROUND = new Color(62, 62, 58);
    private Image[] imgBall, imgHighlightedBall;
    private int[][] iHighlighted;
    private ImageObserver imgObserver;
    private java.util.List<PointInfo> iPrevHighlights;

    // canvas initialization
    public myCanvas(SameGame parent) {
      
      iParent = parent;
      iHighlighted = new int[iParent.iBoard.rows()][iParent.iBoard.columns()];
      addMouseListener(new MouseHandler(this));
      addMouseMotionListener(new MouseMotionHandler(this));
      imgObserver = new FlickerFreeImageObserver();
      setDoubleBuffered(true); // important for flicker-free!
      
      // load and cacheing images
      int i;
      imgBall = new Image[5];
      imgHighlightedBall = new Image[5];
      for (i = 0; i < 5; ++i) {
        imgBall[i] = Toolkit.getDefaultToolkit().getImage("ball" + (i+1) + ".png");
	imgHighlightedBall[i] = Toolkit.getDefaultToolkit().createImage("ball" + (i+1) + "_h.png");
        Toolkit.getDefaultToolkit().prepareImage(imgBall[i], -1, -1, imgObserver);
        Toolkit.getDefaultToolkit().prepareImage(imgHighlightedBall[i], -1, -1, imgObserver);
      }
      randomizeBalls();
    }

    // randomize the ball types
    public void randomizeBalls()
    {
      int i, j;
      Image temp;
      java.util.Random r = new java.util.Random();
      for (i = 0; i < imgBall.length; ++i) {
	j = r.nextInt(imgBall.length);
	temp = imgBall[j];
	imgBall[j] = imgBall[i];
	imgBall[i] = temp;
	temp = imgHighlightedBall[j];
	imgHighlightedBall[j] = imgHighlightedBall[i];
	imgHighlightedBall[i] = temp;
      }
    }

    public void resetHighlights()
    {
      iHighlighted = new int[iParent.iBoard.rows()][iParent.iBoard.columns()];
    }
    
    public void paint(Graphics g)
    {
      if (!(g instanceof Graphics2D))
	return;
      Graphics2D gg = (Graphics2D) g;

      Board board = iParent.iBoard;
      Dimension d = getSize();
      gg.setColor(COLOR_BACKGROUND);
      gg.fillRect(0,0, d.width, d.height);

      Insets insets = getInsets();
      int rowWid = (d.height - insets.top - insets.bottom) / board.rows();
      int colWid = (d.width - insets.left - insets.right) / board.columns();
      int wid = rowWid < colWid ? rowWid : colWid;
      Image imgb; gg.setColor(COLOR_HIGHLIGHTED_BACKGROUND);
      for (int row = 0; row < board.rows(); ++row) {
	for (int col = 0; col < board.columns(); ++col) {
	  int x = insets.left + col * wid;
	  int y = d.height - insets.bottom - row * wid - wid;
	  switch (board.get(row, col)) {
	  case 0:
	  default:
	    continue;
	  case 1:
	  case 2:
	  case 3:
	  case 4:
	  case 5:
	    if (iHighlighted[row][col] > 0) {
	      gg.fillRect(x, y, wid, wid);
	      imgb = imgHighlightedBall[board.get(row, col) - 1];
	    } else
	      imgb = imgBall[board.get(row, col) - 1];
	    break;
	  }
	  gg.drawImage(imgb, x + 4, y + 4, wid - 8, wid - 8, this);
	}
      }
    }

    public void click(int x, int y)
    {
      Board board = iParent.iBoard;
      Dimension d = getSize();
      Insets insets = getInsets();
      int rowWid = (d.height - insets.top - insets.bottom) / board.rows();
      int colWid = (d.width - insets.left - insets.right) / board.columns();
      int wid = rowWid < colWid ? rowWid : colWid;
      int row = (d.height - insets.bottom - y) / wid;
      int col = (x - insets.left) / wid;
      java.util.List<PointInfo> h;
      Iterator<PointInfo> it;
      
      if (0 <= row && row < board.rows() &&
	  0 <= col && col < board.columns())
      {
	iParent.iPoints += board.pick(row, col);
        iPrevRow = row; iPrevCol = col;
      }
      // apply highlights
      resetHighlights();
      h = board.getGroup(row, col);
      it = h.iterator();
      while (it.hasNext()) {
	PointInfo p = it.next();
	iHighlighted[p.row][p.col] = 1;
      }
      iPrevHighlights = h;
      iParent.doUpdate();

      // process when game finished
      if (board.isFinished() && iParent.isPlaying)
	iParent.finishGame();
    }

    public void mouse_moved(int x, int y)
    {
      Board board = iParent.iBoard;
      Dimension d = getSize();
      Insets insets = getInsets();
      int rowWid = (d.height - insets.top - insets.bottom) / board.rows();
      int colWid = (d.width - insets.left - insets.right) / board.columns();
      int wid = rowWid < colWid ? rowWid : colWid;
      int row = (d.height - insets.bottom - y) / wid;
      int col = (x - insets.left) / wid;
      int drawX, drawY;
      boolean isInHighlight = false;
      java.util.Iterator<PointInfo> it;
      Graphics g = this.getGraphics();
      
      // check if neccessary to draw
      if (iPrevRow == row && iPrevCol == col) return;
      if (board.get(row, col) == 0) return;
      if (iHighlighted[row][col] > 0) return;
      
      // check if mouse cursor is still in the same group of balls
      if (iPrevHighlights != null) {
	it = iPrevHighlights.iterator();
	while (it.hasNext()) {
	  PointInfo p =it.next();
	  if (row == p.row && col == p.col) { isInHighlight = true; break; }
	}
      }

      // get the ball group to be highlighted
      java.util.List<PointInfo> s = board.getGroup(row, col);
      
      // draw highlighted balls
      g.setColor(COLOR_HIGHLIGHTED_BACKGROUND);
      it = s.iterator();
      while (it.hasNext()) {
	PointInfo p = it.next();
	drawX = insets.left + p.col * wid;
	drawY = d.height - insets.bottom - p.row * wid - wid;
	g.fillRect(drawX, drawY, wid, wid);
	g.drawImage(imgHighlightedBall[p.ballType - 1], drawX + 4, drawY + 4, wid - 8, wid - 8, imgObserver);
	iHighlighted[p.row][p.col] = 1;
      }
      
      // turn the previously highlighted balls back
      if (iPrevHighlights != null && !isInHighlight) {
	g.setColor(COLOR_BACKGROUND);
	it = iPrevHighlights.iterator();
	while (it.hasNext()) {
	  PointInfo p = it.next();
	  drawX = insets.left + p.col * wid;
	  drawY = d.height - insets.bottom - p.row * wid - wid;
	  g.fillRect(drawX, drawY, wid, wid);
	  if (board.get(p.row, p.col) != 0) {
	    // draw the current state (some balls may be disapeared :)
	    g.drawImage(imgBall[board.get(p.row, p.col) - 1], drawX + 4, drawY + 4, wid - 8, wid - 8, imgObserver);
	    iHighlighted[p.row][p.col] = 0;
	  }
	}
      }
      // save current information not to do duplicated works
      iPrevHighlights = s;
      iPrevRow = row;
      iPrevCol = col;
    }

    static class MouseHandler extends java.awt.event.MouseAdapter {
      private myCanvas iParent;
      public MouseHandler(myCanvas g) { iParent = g; }
      
      public void mouseClicked(MouseEvent e)
      {
	iParent.click(e.getX(), e.getY());
      }
    }

    static class MouseMotionHandler extends java.awt.event.MouseMotionAdapter {
      private myCanvas iParent;
      public MouseMotionHandler(myCanvas g) { iParent = g; }

      public void mouseDragged(MouseEvent e) {}
      public void mouseMoved(MouseEvent e)
      {
	iParent.mouse_moved(e.getX(), e.getY());
      }
    }

    static class FlickerFreeImageObserver implements java.awt.image.ImageObserver {
      public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
        return !((infoflags & ImageObserver.ALLBITS) == ImageObserver.ALLBITS);
      }
    }
  }

  // --------------------------------------------------------------------
  //
  // The Main Method

  public static void main(String [] args)
  {
    if (args.length != 0 && args.length != 2 && args.length != 3) {
      System.err.println("Usage: java SameGame [<rows> <columns> [<# of balltypes>]]");
      System.exit(1);
    }
    
    GameOption.load();
    try { // override defaults
      if (args.length >= 2) {
	GameOption.setNumOfRows(Integer.parseInt(args[0]));
	GameOption.setNumOfCols(Integer.parseInt(args[1]));
	if (args.length == 3) {
	  GameOption.setNumOfBallTypes(Integer.parseInt(args[2]));
	}
      }
    } catch (OutOfRangeException e) {
      System.err.println("Out of range in your options.");
      System.err.println("<rows> : " + GameOption.minRows + " ~ " + GameOption.maxRows);
      System.err.println("<cols> : " + GameOption.minCols + " ~ " + GameOption.maxCols);
      System.err.println("<# of balltypes> : "+ GameOption.minBallTypes + " ~ " + GameOption.maxBallTypes);
      System.exit(1);
    }

    Ranking.load();

    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) { }

    Board board = new Board(GameOption.getNumOfRows(), GameOption.getNumOfCols(), GameOption.getNumOfBallTypes());
    SameGame game = new SameGame(board);
    game.setVisible(true);
  }

  // --------------------------------------------------------------------
}

/* vim: set ts=8 sw=2 sts=2 noet: */
